import logging

from avtomat_aws.decorators.authenticate import authenticate
from avtomat_aws.decorators.set_logger import set_logger
from avtomat_aws.decorators.validate import validate
from avtomat_aws.helpers.set_session_objects import set_session_objects

logger = logging.getLogger(__name__)

DEFAULTS = {"region": None, "debug": False, "silent": False}
RULES = [
    {"required": ["snapshot_ids", "target_account"]},
    {
        "choice": [
            {"type": ["gp2", "gp3", "io1", "io2", "st1", "sc1", "standard"]},
        ]
    },
    {"at_least_one": ["size", "type", "iops"]},
]


@validate(DEFAULTS, RULES)
@set_logger()
@authenticate()
def share_snapshots(**kwargs):
    """Share EC2 snapshots with other accounts"""

    # Required parameters
    snapshot_ids = kwargs.pop("snapshot_ids")
    target_account = kwargs.pop("target_account")

    if not snapshot_ids:
        logger.info("No snapshots to share")
        return []

    session_objects = set_session_objects(
        kwargs["session"], clients=["ec2"], region=kwargs["region"]
    )

    logger.info(f"Sharing snapshots with account {target_account}")

    snapshots = []
    for snapshot_id in snapshot_ids:
        try:
            session_objects["ec2_client"].modify_snapshot_attribute(
                Attribute="createVolumePermission",
                CreateVolumePermission={"Add": [{"UserId": target_account}]},
                SnapshotId=snapshot_id,
                OperationType="add",
            )
            snapshots.append(snapshot_id)
            logger.info(f"{snapshot_id} - shared")
        except Exception as e:
            logger.error(f"{snapshot_id} - failed to share snapshot - {e}")
            continue

    logger.info(f"{len(snapshots)} snapshots shared")
    logger.debug(snapshots)

    return snapshots



import logging

from avtomat_aws.decorators.authenticate import authenticate
from avtomat_aws.decorators.set_logger import set_logger
from avtomat_aws.decorators.validate import validate
from avtomat_aws.helpers.set_session_objects import set_session_objects

logger = logging.getLogger(__name__)

DEFAULTS = {"region": None, "debug": False, "silent": False}


@validate(DEFAULTS)
@set_logger()
@authenticate()
def discover_no_mfa_users(**kwargs):
    """Discover IAM users without MFA enabled"""

    session_objects = set_session_objects(
        kwargs["session"], clients=["iam"], region=kwargs["region"]
    )

    logger.info(f"Discovering users without MFA enabled")

    users = []
    users_response = session_objects["iam_client"].list_users(MaxItems=50)
    while True:
        for user in users_response["Users"]:
            try:
                mfa_response = session_objects["iam_client"].list_mfa_devices(
                    UserName=user["UserName"]
                )
            except Exception as e:
                logger.error(f"Failed to list MFA devices for {user['UserName']} - {e}")
                continue
            if not mfa_response["MFADevices"]:
                users.append({"UserName": user["UserName"]})
        if users_response["IsTruncated"]:
            users_response = session_objects["iam_client"].list_users(
                Marker=users_response["Marker"]
            )
        else:
            break

    logger.info(f"{len(users)} users found")
    logger.debug(users)

    return users



import logging
import time

from avtomat_aws.decorators.authenticate import authenticate
from avtomat_aws.decorators.set_logger import set_logger
from avtomat_aws.decorators.validate import validate
from avtomat_aws.helpers.set_session_objects import set_session_objects

logger = logging.getLogger(__name__)

DEFAULTS = {
    "region": None,
    "pending_limit": 20,
    "encrypt": False,
    "kms_key_id": None,
    "debug": False,
    "silent": False,
}
RULES = [{"and": ["encrypt", "kms_key_id"]}]


@validate(DEFAULTS, RULES)
@set_logger()
@authenticate()
def copy_snapshots(**kwargs):
    """Move EC2 snapshots between regions or accounts"""

    # Required parameters
    target_region = kwargs.pop("target_region")
    snapshot_ids = kwargs.pop("snapshot_ids")

    if not snapshot_ids:
        logger.info("No snapshots to copy")
        return []

    session_objects = set_session_objects(
        kwargs["session"], clients=["ec2"], region=target_region
    )

    logger.info(f"Creating snapshot copies in {target_region}")

    snapshots = []
    pending_snapshots = set()
    pending_limit = kwargs["pending_limit"]
    logger.debug(f"Custom pending limit: {pending_limit}")

    for snapshot_id in snapshot_ids:
        while len(pending_snapshots) >= pending_limit:
            logger.debug(f"Reached {pending_limit} pending snapshots, waiting...")
            for snapshot in list(pending_snapshots):
                status = session_objects["ec2_client"].describe_snapshots(
                    SnapshotIds=[snapshot]
                )["Snapshots"][0]["State"]
                if status != "pending":
                    pending_snapshots.remove(snapshot)
            time.sleep(30)

        try:
            copy_args = {
                "SourceRegion": kwargs["region"],
                "SourceSnapshotId": snapshot_id,
                "Description": f"Copy of {snapshot_id} in {kwargs['region']}",
            }

            if kwargs.get("kms_key_id"):
                copy_args["KmsKeyId"] = kwargs["kms_key_id"]
            if kwargs.get("encrypt"):
                copy_args["Encrypted"] = True

            response = session_objects["ec2_client"].copy_snapshot(**copy_args)
            logger.info(f"{snapshot_id} - copying as {response['SnapshotId']}")
            pending_snapshots.add(response["SnapshotId"])
            snapshots.append(response["SnapshotId"])
        except Exception as e:
            logger.error(f"{snapshot_id} - failed to copy snapshot - {e}")
            continue

    logger.info(f"{len(snapshots)} snapshots copied")
    logger.debug(snapshots)

    return snapshots



from avtomat_aws.services.general import get_date

ACTION_DESCRIPTION = "Return a date in the requested format."


def add_cli_arguments(parser):
    """Argument parsing"""

    parser.add_argument(
        "--after",
        type=int,
        help="Return a date number of days in the future.",
        required=False,
    )
    parser.add_argument(
        "--before",
        type=int,
        help="Return a date number of days in the past.",
        required=False,
    )
    parser.add_argument(
        "--format", help="Return the date in this format.", required=False
    )


def cli(args):
    """Command-line interface function"""

    inputs = vars(args)

    try:
        result = get_date(**inputs)
        print(result)
    except Exception as e:
        print(f"Action failed - {e}")
        exit(1)



import logging

from avtomat_aws.decorators.authenticate import authenticate
from avtomat_aws.decorators.set_logger import set_logger
from avtomat_aws.decorators.validate import validate
from avtomat_aws.helpers.set_session_objects import set_session_objects

logger = logging.getLogger(__name__)

DEFAULTS = {
    "enable": False,
    "disable": False,
    "region": None,
    "debug": False,
    "silent": False,
}
RULES = [
    {"required": ["keys"]},
    {"at_least_one": ["enable", "disable"]},
    {"at_most_one": ["enable", "disable"]},
]


@validate(DEFAULTS, RULES)
@set_logger()
@authenticate()
def modify_access_keys(**kwargs):
    """Disable specific access keys"""

    # Required parameters
    keys = kwargs.pop("keys")

    if not keys:
        logger.info("No access keys to modify")
        return []

    session_objects = set_session_objects(
        kwargs["session"], clients=["iam"], region=kwargs["region"]
    )

    if kwargs.get("disable"):
        logger.info(f"Disabling access keys")
        status = "Inactive"
    elif kwargs.get("enable"):
        logger.info(f"Enabling access keys")
        status = "Active"
    else:
        raise ValueError("No action specified")
    logger.debug(f"Keys: {keys}")

    failed = []
    counter = 0
    for key in keys:
        access_key_id = key["AccessKeyId"]
        username = key["UserName"]

        try:
            session_objects["iam_client"].update_access_key(
                AccessKeyId=access_key_id, Status=status, UserName=username
            )
            logger.debug(f"{access_key_id} - {status}")
            counter += 1
        except Exception as e:
            failed.append(access_key_id)
            logger.error(f"{access_key_id} - failed to update - {e}")
            continue

    if kwargs.get("disable"):
        logger.info(f"Disabled {counter} access keys")
    elif kwargs.get("enable"):
        logger.info(f"Enabled {counter} access keys")
    if failed:
        logger.warning(f"{len(failed)} access keys failed to update")
        logger.debug(failed)

    return failed



from avtomat_aws.services.ec2 import delete_security_groups

ACTION_DESCRIPTION = "Delete EC2 security groups."


def add_cli_arguments(parser):
    """Argument parsing"""

    required = parser.add_argument_group("required")

    required.add_argument(
        "--security_group_ids",
        nargs="+",
        help="Security Group IDs to delete, separated by space.",
        required=True,
    )


def cli(args):
    """Command-line interface function"""

    inputs = vars(args)

    try:
        result = delete_security_groups(**inputs)
        for item in result:
            print(item)
    except Exception as e:
        print(f"Action failed - {e}")
        exit(1)



import logging
import os

from avtomat_aws.decorators.authenticate import authenticate
from avtomat_aws.decorators.set_logger import set_logger
from avtomat_aws.decorators.validate import validate
from avtomat_aws.helpers.set_session_objects import set_session_objects

logger = logging.getLogger(__name__)

DEFAULTS = {"prefix": "", "region": None, "debug": False, "silent": False}
RULES = [{"required": ["bucket", "objects"]}]


@validate(DEFAULTS, RULES)
@set_logger()
@authenticate()
def create_objects(**kwargs):
    """Create objects in an S3 bucket"""

    # Required parameters
    bucket = kwargs.pop("bucket")
    objects = kwargs.pop("objects")

    if not objects:
        logger.info("No objects to create")
        return []

    session_objects = set_session_objects(
        kwargs["session"], clients=["s3"], region=kwargs["region"]
    )

    logger.info(f"Creating objects in '{bucket}'")

    failed = []
    counter = 0
    for obj in objects:
        key = obj["Key"]
        body = obj["Body"]
        if kwargs.get("prefix"):
            prefix = kwargs["prefix"].rstrip("/")
            key = f"{prefix}/{key}"

        if os.path.isfile(obj["Body"]):
            try:
                with open(body, "rb") as f:
                    body = f.read()
            except Exception as e:
                failed.append(key)
                logger.error(f"{key} - failed to read {body} - {e}")
                continue
        try:
            session_objects["s3_client"].put_object(Bucket=bucket, Key=key, Body=body)
            counter += 1
            logger.debug(f"{key} - created")
        except Exception as e:
            failed.append(key)
            logger.error(f"{key} - failed to create - {e}")

    logger.info(f"{counter} objects created")
    if failed:
        logger.warning(f"{len(failed)} objects failed to create")
        logger.warning(failed)

    return failed



from avtomat_aws.services.sts import create_session

ACTION_DESCRIPTION = "Create an authenticated session."


def print_export_commands(session):
    """Print credentials for CLI"""

    credentials = session.get_credentials()
    current_credentials = credentials.get_frozen_credentials()

    print(f"export AWS_ACCESS_KEY_ID={current_credentials.access_key}")
    print(f"export AWS_SECRET_ACCESS_KEY={current_credentials.secret_key}")
    print(f"export AWS_SESSION_TOKEN={current_credentials.token}")


def add_cli_arguments(parser):
    """Argument parsing"""

    parser.add_argument("--access_key", help="AWS Access Key", required=False)
    parser.add_argument("--secret_key", help="AWS Secret Key", required=False)
    parser.add_argument("--session_token", help="AWS Session Token", required=False)
    parser.add_argument("--profile", help="AWS Profile", required=False)
    parser.add_argument("--role_arn", help="Role ARN to assume", required=False)
    parser.add_argument("--mfa_serial", help="MFA Serial Number", required=False)
    parser.add_argument("--mfa_token", help="MFA Token", required=False)


def cli(args):
    """Command-line interface function"""

    inputs = vars(args)

    try:
        result = create_session(**inputs)
        print_export_commands(result)
    except Exception as e:
        print(f"Action failed - {e}")
        exit(1)



from avtomat_aws.services.backup import create_backups

ACTION_DESCRIPTION = "Start an on-demand backup job for the specified resources."


def add_cli_arguments(parser):
    """Argument parsing"""

    required = parser.add_argument_group("required")

    required.add_argument(
        "--resource_ids",
        nargs="+",
        help="Resource IDs to create backups for, separated by space.",
        required=True,
    )
    required.add_argument(
        "--service", help="Service to which the resource IDs belong.", required=True
    )
    parser.add_argument(
        "--backup_vault_name",
        help="Backup vault in which to store the backups.",
        required=False,
        default="Default",
    )
    parser.add_argument(
        "--retention_days",
        type=int,
        help="Number of days to retain the backups.",
        required=False,
    )
    parser.add_argument(
        "--iam_role",
        help="Role ARN used to create the backup. Leave blank for AWS Backup Default Service Role.",
        required=False,
    )
    parser.add_argument(
        "--wait",
        action="store_true",
        help="Wait for backups to complete before proceeding.",
        required=False,
    )


def cli(args):
    """Command-line interface function"""

    inputs = vars(args)

    try:
        result = create_backups(**inputs)
        for item in result:
            print(item)
    except Exception as e:
        print(f"Action failed - {e}")
        exit(1)



from avtomat_aws.services.ec2 import discover_no_ssm_instances

ACTION_DESCRIPTION = "Discover EC2 instances without SSM enabled."


def add_cli_arguments(parser):
    """Argument parsing"""

    parser.add_argument(
        "--instance_ids",
        nargs="+",
        help="Instance IDs to focus on, separated by space.",
        required=False,
    )


def cli(args):
    """Command-line interface function"""

    inputs = vars(args)

    try:
        result = discover_no_ssm_instances(**inputs)
        for item in result:
            print(item)
    except Exception as e:
        print(f"Action failed - {e}")
        exit(1)



from avtomat_aws.services.s3 import create_objects

ACTION_DESCRIPTION = "Create objects in an S3 bucket."


def add_cli_arguments(parser):
    """Argument parsing"""

    required = parser.add_argument_group("required")

    required.add_argument(
        "--bucket", help="Name of the S3 bucket to create objects in.", required=True
    )
    required.add_argument(
        "--object",
        help="Object with Key and Body. Repeatable for multiple objects.",
        action="append",
        nargs=2,
        metavar=("KEY", "BODY"),
        required=True,
    )
    parser.add_argument(
        "--prefix", help="Prefix for path to create objects under.", required=False
    )


def cli(args):
    """Command-line interface function"""

    objects = [{"Key": obj[0], "Body": obj[1]} for obj in args.object]

    inputs = {
        "bucket": args.bucket,
        "objects": objects,
        "prefix": args.prefix,
        "region": args.region,
        "debug": args.debug,
        "silent": args.silent,
    }

    try:
        result = create_objects(**inputs)
        for item in result:
            print(item)
    except Exception as e:
        print(f"Action failed - {e}")
        exit(1)



import logging

from avtomat_aws.decorators.authenticate import authenticate
from avtomat_aws.decorators.set_logger import set_logger
from avtomat_aws.decorators.validate import validate
from avtomat_aws.helpers.set_session_objects import set_session_objects

logger = logging.getLogger(__name__)

DEFAULTS = {
    "instance_ids": [],
    "volume_ids": [],
    "unencrypted": False,
    "detached": False,
    "types": None,
    "root": False,
    "region": None,
    "debug": False,
    "silent": False,
}
RULES = [
    {
        "choice": [
            {"types": ["gp2", "gp3", "io1", "io2", "st1", "sc1", "standard"]},
        ]
    },
    {"at_most_one": ["instance_ids", "volume_ids"]},
]


@validate(DEFAULTS, RULES)
@set_logger()
@authenticate()
def discover_volumes(**kwargs):
    """Discover EBS volumes based on provided criteria"""

    logger.info("Discovering volumes")

    filters = build_filters(**kwargs)
    volumes = search_volumes(filters, **kwargs)

    logger.info(f"{len(volumes)} volumes found")
    logger.debug(volumes)

    return volumes


def build_filters(**kwargs):
    """Construct filters"""

    instance_ids = kwargs.get("instance_ids")
    volume_ids = kwargs.get("volume_ids")
    unencrypted = kwargs.get("unencrypted")
    detached = kwargs.get("detached")
    types = kwargs.get("types")
    root = kwargs.get("root")

    filters = []
    if instance_ids:
        logger.debug(f"Targeting instance volumes: {instance_ids}")
        filters.append({"Name": "attachment.instance-id", "Values": instance_ids})
    elif volume_ids:
        logger.debug(f"Targeting specific volumes: {volume_ids}")
    else:
        logger.debug("Targeting all volumes")
    if unencrypted:
        logger.debug("Filtering for unencrypted volumes")
        filters.append({"Name": "encrypted", "Values": ["false"]})
    if detached:
        logger.debug("Filtering for detached volumes")
        filters.append({"Name": "status", "Values": ["available"]})
    if types:
        logger.debug(f"Filtering for types: {types}")
        filters.append({"Name": "volume-type", "Values": types})
    if root:
        logger.debug("Filtering for root volumes")
        root_devices = get_root_devices(**kwargs)
        filters.append({"Name": "attachment.device", "Values": root_devices})

    return filters


def search_volumes(filters, **kwargs):
    """Search for volumes in specified region"""

    session = kwargs["session"]
    region = kwargs["region"]
    volume_ids = kwargs.get("volume_ids")

    session_objects = set_session_objects(session, resources=["ec2"], region=region)
    volumes = []

    response = session_objects["ec2_resource"].volumes.filter(
        Filters=filters, VolumeIds=volume_ids
    )

    for volume in response:
        volumes.append(volume.id)

    return volumes


def get_root_devices(**kwargs):
    """Get root devices for specified instances"""

    session = kwargs["session"]
    region = kwargs["region"]
    instance_ids = kwargs.get("instance_ids")

    session_objects = set_session_objects(session, resources=["ec2"], region=region)

    root_devices = []
    for instance in session_objects["ec2_resource"].instances.filter(
        InstanceIds=instance_ids
    ):
        root_devices.append(instance.root_device_name)

    return list(set(root_devices))



import logging
from datetime import datetime, timedelta

from avtomat_aws.decorators.authenticate import authenticate
from avtomat_aws.decorators.set_logger import set_logger
from avtomat_aws.decorators.validate import validate
from avtomat_aws.helpers.set_session_objects import set_session_objects

logger = logging.getLogger(__name__)

DEFAULTS = {
    "events": [],
    "created_after": (datetime.now() - timedelta(days=90)).strftime("%Y/%m/%d"),
    "created_before": (datetime.now()).strftime("%Y/%m/%d"),
    "region": None,
    "debug": False,
    "silent": False,
}
RULES = [
    {"required": ["resource_id"]},
    {"date_yymmdd": ["created_after", "created_before"]},
    {"cloudtrail": ["created_after", "created_before"]},
]


@validate(DEFAULTS, RULES)
@set_logger()
@authenticate()
def discover_resource_events(**kwargs):
    """Discover events for a specific resource"""

    # Required parameters
    resource_id = kwargs.pop("resource_id")

    if kwargs.get("created_after"):
        kwargs["created_after"] = datetime.strptime(kwargs["created_after"], "%Y/%m/%d")
    if kwargs.get("created_before"):
        kwargs["created_before"] = datetime.strptime(
            kwargs["created_before"], "%Y/%m/%d"
        )

    logger.info(f"Discovering CloudTrail events for resource '{resource_id}'")

    unfiltered_events = search_events(resource_id, **kwargs)
    events = filter_events(unfiltered_events, **kwargs)

    logger.info(f"{len(events)} events found")
    logger.debug(events)

    return events


def search_events(resource_id, **kwargs):
    """Search for events"""

    session = kwargs["session"]
    region = kwargs["region"]
    created_after = kwargs.get("created_after")
    created_before = kwargs.get("created_before")

    session_objects = set_session_objects(
        session, clients=["cloudtrail"], region=region
    )
    events = []

    response = session_objects["cloudtrail_client"].lookup_events(
        LookupAttributes=[
            {"AttributeKey": "ResourceName", "AttributeValue": resource_id}
        ],
        StartTime=created_after,
        EndTime=created_before,
    )

    while True:
        for event in response["Events"]:
            data = {
                "UserName": event["Username"],
                "EventTime": event["EventTime"].isoformat(),
                "EventName": event["EventName"],
                "Resources": [
                    resource["ResourceName"] for resource in event["Resources"]
                ],
            }
            events.append(data)

        if response.get("NextToken"):
            response = session_objects["cloudtrail_client"].lookup_events(
                LookupAttributes=[
                    {"AttributeKey": "ResourceName", "AttributeValue": resource_id}
                ],
                StartTime=created_after,
                EndTime=created_before,
                NextToken=response["NextToken"],
            )
        else:
            break

    return events


def filter_events(unfiltered_events, **kwargs):
    """Filter events based on specified criteria"""

    events = kwargs.get("events")
    filtered_events = unfiltered_events

    if events:
        logger.debug(f"Filtering for events: {events}")
        filtered_events = [
            event for event in filtered_events if event["EventName"] in events
        ]

    return filtered_events



from avtomat_aws.services.s3 import delete_objects

ACTION_DESCRIPTION = "Delete objects from an S3 bucket."


def add_cli_arguments(parser):
    """Argument parsing"""

    required = parser.add_argument_group("required")

    required.add_argument(
        "--bucket", help="Name of the S3 bucket to delete objects from.", required=True
    )
    required.add_argument(
        "--objects",
        nargs="+",
        help="Objects to delete, separated by space.",
        required=True,
    )
    parser.add_argument("--prefix", help="Prefix to filter objects by.", required=False)


def cli(args):
    """Command-line interface function"""

    inputs = vars(args)

    try:
        result = delete_objects(**inputs)
        for item in result:
            print(item)
    except Exception as e:
        print(f"Action failed - {e}")
        exit(1)



import logging

from botocore.config import Config

logger = logging.getLogger(__name__)


config = Config(retries={"max_attempts": 5, "mode": "standard"})


def set_session_objects(session, region, clients=None, resources=None):
    """Set the session objects(clients and resources)."""

    session_objects = {}

    if clients:
        for client in clients:
            session_objects[f"{client}_client"] = session.client(
                client, region_name=region, config=config
            )

    if resources:
        for resource in resources:
            session_objects[f"{resource}_resource"] = session.resource(
                resource, region_name=region, config=config
            )

    return session_objects



import logging
from datetime import datetime, timezone

from avtomat_aws.decorators.authenticate import authenticate
from avtomat_aws.decorators.set_logger import set_logger
from avtomat_aws.decorators.validate import validate
from avtomat_aws.helpers.set_session_objects import set_session_objects

logger = logging.getLogger(__name__)

DEFAULTS = {"region": None, "debug": False, "silent": False}
RULES = [{"required": ["threshold_days"]}]


@validate(DEFAULTS, RULES)
@set_logger()
@authenticate()
def discover_inactive_users(**kwargs):
    """Discover IAM users who haven't used the console and any access keys over a certain period"""

    # Required parameters
    threshold_days = kwargs.pop("threshold_days")

    session_objects = set_session_objects(
        kwargs["session"], clients=["iam"], region=kwargs["region"]
    )

    logger.info(f"Discovering users with last activity over {threshold_days} days")

    users = []
    users_response = session_objects["iam_client"].list_users(MaxItems=50)
    while True:
        for user in users_response["Users"]:
            try:
                access_keys_recently_used = check_access_keys_recently_used(
                    user, threshold_days, session_objects
                )
                password_recently_used = check_password_recently_used(
                    user, threshold_days, session_objects
                )
            except Exception as e:
                logger.error(
                    f"Failed to check user activity for {user['UserName']} - {e}"
                )
                continue

            if not access_keys_recently_used and not password_recently_used:
                users.append({"UserName": user["UserName"]})

        if users_response["IsTruncated"]:
            users_response = session_objects["iam_client"].list_users(
                Marker=users_response["Marker"]
            )
        else:
            break

    logger.info(f"{len(users)} users found")
    logger.debug(users)

    return users


def check_access_keys_recently_used(user, threshold_days, session_objects):
    """Check if user has used any access keys within the threshold days"""

    keys_response = session_objects["iam_client"].list_access_keys(
        UserName=user["UserName"]
    )

    for key in keys_response["AccessKeyMetadata"]:
        last_used = session_objects["iam_client"].get_access_key_last_used(
            AccessKeyId=key["AccessKeyId"]
        )

        if "LastUsedDate" in last_used["AccessKeyLastUsed"]:
            age = (
                datetime.now(timezone.utc)
                - last_used["AccessKeyLastUsed"]["LastUsedDate"]
            ).days
            if age < threshold_days:
                return True

    return False


def check_password_recently_used(user, threshold_days, session_objects):
    """Check if user has used their password within the threshold days"""

    password_last_used = (
        session_objects["iam_client"]
        .get_user(UserName=user["UserName"])["User"]
        .get("PasswordLastUsed")
    )

    if password_last_used:
        age = (datetime.now(timezone.utc) - password_last_used).days
        if age < threshold_days:
            return True

    return False



import logging
from datetime import datetime, timedelta

from avtomat_aws.decorators.authenticate import authenticate
from avtomat_aws.decorators.set_logger import set_logger
from avtomat_aws.decorators.validate import validate
from avtomat_aws.helpers.set_session_objects import set_session_objects

logger = logging.getLogger(__name__)

DEFAULTS = {
    "created_after": (datetime.now() - timedelta(days=90)).strftime("%Y/%m/%d"),
    "created_before": (datetime.now()).strftime("%Y/%m/%d"),
    "region": None,
    "debug": False,
    "silent": False,
}
RULES = [
    {"required": ["event"]},
    {"date_yymmdd": ["created_after", "created_before"]},
    {"cloudtrail": ["created_after", "created_before"]},
]


@validate(DEFAULTS, RULES)
@set_logger()
@authenticate()
def discover_events(**kwargs):
    """Discover events by name"""

    # Required parameters
    event = kwargs.pop("event")

    if kwargs.get("created_after"):
        kwargs["created_after"] = datetime.strptime(kwargs["created_after"], "%Y/%m/%d")
    if kwargs.get("created_before"):
        kwargs["created_before"] = datetime.strptime(
            kwargs["created_before"], "%Y/%m/%d"
        )

    logger.info(f"Discovering CloudTrail events for '{event}'")

    events = search_events(event, **kwargs)

    logger.info(f"{len(events)} events found")
    logger.debug(events)

    return events


def search_events(event, **kwargs):
    """Search for events"""

    session = kwargs["session"]
    region = kwargs["region"]
    created_after = kwargs.get("created_after")
    created_before = kwargs.get("created_before")

    session_objects = set_session_objects(
        session, clients=["cloudtrail"], region=region
    )
    events = []

    response = session_objects["cloudtrail_client"].lookup_events(
        LookupAttributes=[{"AttributeKey": "EventName", "AttributeValue": event}],
        StartTime=created_after,
        EndTime=created_before,
    )

    while True:
        for obj in response["Events"]:
            data = {
                "UserName": obj["Username"],
                "EventTime": obj["EventTime"].isoformat(),
                "EventName": obj["EventName"],
                "Resources": [
                    resource["ResourceName"] for resource in obj["Resources"]
                ],
            }
            events.append(data)

        if response.get("NextToken"):
            response = session_objects["cloudtrail_client"].lookup_events(
                LookupAttributes=[
                    {"AttributeKey": "EventName", "AttributeValue": event}
                ],
                StartTime=created_after,
                EndTime=created_before,
                NextToken=response["NextToken"],
            )
        else:
            break

    return events



from .discover_inactive_console_users import discover_inactive_console_users
from .discover_inactive_users import discover_inactive_users
from .discover_no_mfa_users import discover_no_mfa_users
from .discover_old_access_keys import discover_old_access_keys
from .discover_old_password_users import discover_old_password_users
from .discover_unused_access_keys import discover_unused_access_keys
from .discover_unused_roles import discover_unused_roles
from .modify_access_keys import modify_access_keys
from .modify_users_console_access import modify_users_console_access
from .quarantine_user import quarantine_user



from avtomat_aws.services.iam import modify_users_console_access

ACTION_DESCRIPTION = "Enable or disable AWS Management Console access for an IAM user."


def add_cli_arguments(parser):
    """Argument parsing"""

    required = parser.add_argument_group("required")

    required.add_argument(
        "--user",
        help="User with Username and Password(optional). Repeatable for multiple keys.",
        action="append",
        nargs="+",
        metavar=("USERNAME", "PASSWORD"),
        required=True,
    )
    parser.add_argument(
        "--enable", action="store_true", help="Enable console access.", required=False
    )
    parser.add_argument(
        "--disable", action="store_true", help="Disable console access.", required=False
    )


def cli(args):
    """Command-line interface function"""

    users = [
        {"UserName": u[0], "Password": u[1] if len(u) > 1 else None} for u in args.user
    ]

    inputs = {
        "users": users,
        "enable": args.enable,
        "disable": args.disable,
        "region": args.region,
        "debug": args.debug,
        "silent": args.silent,
    }

    try:
        result = modify_users_console_access(**inputs)
        for item in result:
            print(item)
    except Exception as e:
        print(f"Action failed - {e}")
        exit(1)



from avtomat_aws.services.iam import modify_access_keys

ACTION_DESCRIPTION = "Enable or disable an IAM access key."


def add_cli_arguments(parser):
    """Argument parsing"""

    required = parser.add_argument_group("required")

    required.add_argument(
        "--key",
        help="Access key with Id and Username. Repeatable for multiple keys.",
        action="append",
        nargs=2,
        metavar=("ID", "USER"),
        required=True,
    )
    parser.add_argument(
        "--enable", action="store_true", help="Enable the access key.", required=False
    )
    parser.add_argument(
        "--disable", action="store_true", help="Disable the access key.", required=False
    )


def cli(args):
    """Command-line interface function"""

    keys = [{"AccessKeyId": key[0], "UserName": key[1]} for key in args.key]

    inputs = {
        "keys": keys,
        "enable": args.enable,
        "disable": args.disable,
        "region": args.region,
        "debug": args.debug,
        "silent": args.silent,
    }

    try:
        result = modify_access_keys(**inputs)
        for item in result:
            print(item)
    except Exception as e:
        print(f"Action failed - {e}")
        exit(1)



import logging

from avtomat_aws.decorators.authenticate import authenticate
from avtomat_aws.decorators.set_logger import set_logger
from avtomat_aws.decorators.validate import validate
from avtomat_aws.helpers.set_session_objects import set_session_objects

logger = logging.getLogger(__name__)

DEFAULTS = {
    "size": None,
    "type": None,
    "iops": None,
    "snapshot": False,
    "region": None,
    "debug": False,
    "silent": False,
}
RULES = [
    {"required": ["volume_ids"]},
    {
        "choice": [
            {"type": ["gp2", "gp3", "io1", "io2", "st1", "sc1", "standard"]},
        ]
    },
    {"at_least_one": ["size", "type", "iops"]},
]


@validate(DEFAULTS, RULES)
@set_logger()
@authenticate()
def modify_volumes(**kwargs):
    """Modify EBS volumes"""

    # Required parameters
    volume_ids = kwargs.pop("volume_ids")

    if not volume_ids:
        logger.info("No volumes to modify")
        return []

    session_objects = set_session_objects(
        kwargs["session"], clients=["ec2"], resources=["ec2"], region=kwargs["region"]
    )

    logger.info(f"Modifying volumes")

    snapshots_to_wait = []
    if kwargs.get("snapshot"):
        for volume_id in volume_ids:
            volume = session_objects["ec2_resource"].Volume(volume_id)
            try:
                snapshot = volume.create_snapshot(
                    Description=f"Backup before modification of {volume_id}",
                    TagSpecifications=[
                        {
                            "ResourceType": "snapshot",
                            "Tags": [{"Key": "Name", "Value": f"source:{volume_id}"}],
                        }
                    ],
                )
                logger.info(f"{volume_id} - creating {snapshot.id}")
                snapshots_to_wait.append(snapshot)
            except Exception as e:
                logger.error(f"{volume_id} - failed to create snapshot - {e}")
                continue

    for snapshot in snapshots_to_wait:
        snapshot.wait_until_completed()

    modified_volumes = []
    for volume_id in volume_ids:
        try:
            modify_args = {"VolumeId": volume_id}
            if kwargs.get("size"):
                modify_args["Size"] = kwargs["size"]
            if kwargs.get("type"):
                modify_args["VolumeType"] = kwargs["type"]
            if kwargs.get("iops"):
                modify_args["Iops"] = kwargs["iops"]
            session_objects["ec2_client"].modify_volume(**modify_args)
            modified_volumes.append(volume_id)
            logger.info(f"{volume_id} - modified")
        except Exception as e:
            logger.error(f"{volume_id} - failed to modify - {e}")

    if kwargs.get("size"):
        logger.info(
            "Ensure the server file system is extended to use the newly increased volume size."
        )

    logger.info(f"{len(modified_volumes)} volumes modified")
    logger.debug(modified_volumes)

    return modified_volumes



import datetime
import logging

from avtomat_aws.decorators.set_logger import set_logger
from avtomat_aws.decorators.validate import validate

logger = logging.getLogger(__name__)

DEFAULTS = {
    "after": None,
    "before": None,
    "format": "datetime",
    "debug": False,
    "silent": False,
}
RULES = [
    {"at_most_one": ["after", "before"]},
    {"choice": [{"format": ["datetime", "string", "iso"]}]},
]


@validate(DEFAULTS, RULES)
@set_logger()
def get_date(**kwargs):
    """Return a date in the requested format."""

    date_format = kwargs.get("format")
    string_format = "%Y/%m/%d"

    # Determine date
    now = datetime.datetime.utcnow()  # UTC
    if kwargs.get("after"):
        logger.info(f"Returning date {kwargs['after']} days in the future")
        delta = datetime.timedelta(days=kwargs["after"])
        date = now + delta
    elif kwargs.get("before"):
        logger.info(f"Returning date {kwargs['before']} days in the past")
        delta = datetime.timedelta(days=-kwargs["before"])
        date = now + delta
    else:
        logger.info("Returning current date")
        date = now

    # Determine output format
    if date_format == "string":
        date = date.strftime(string_format)
    elif date_format == "iso":
        date = date.isoformat()
    else:
        date = date

    logger.info(f"Date: {date}")

    return date



from avtomat_aws.services.backup import delete_backups

ACTION_DESCRIPTION = "Delete backup recovery points."


def add_cli_arguments(parser):
    """Argument parsing"""

    required = parser.add_argument_group("required")

    required.add_argument(
        "--backup_vault_name",
        help="Backup vault from which to delete recovery points.",
        required=True,
    )
    required.add_argument(
        "--recovery_point_arns",
        nargs="+",
        help="Recovery point ARNs to delete, separated by space.",
        required=True,
    )


def cli(args):
    """Command-line interface function"""

    inputs = vars(args)

    try:
        result = delete_backups(**inputs)
        for item in result:
            print(item)
    except Exception as e:
        print(f"Action failed - {e}")
        exit(1)



import logging

from avtomat_aws.decorators.authenticate import authenticate
from avtomat_aws.decorators.set_logger import set_logger
from avtomat_aws.decorators.validate import validate
from avtomat_aws.helpers.set_session_objects import set_session_objects

logger = logging.getLogger(__name__)

DEFAULTS = {
    "states": ["running", "stopped", "pending", "stopping"],
    "tags": [],
    "instance_ids": [],
    "invert": False,
    "os": None,
    "public": False,
    "region": None,
    "debug": False,
    "silent": False,
}
RULES = [
    {
        "choice": [
            {
                "states": [
                    "pending",
                    "running",
                    "shutting-down",
                    "terminated",
                    "stopping",
                    "stopped",
                ]
            },
            {"os": ["linux", "windows"]},
        ]
    }
]


@validate(DEFAULTS, RULES)
@set_logger()
@authenticate()
def discover_instances(**kwargs):
    """Discover EC2 instances based on provided criteria"""

    session_objects = set_session_objects(
        kwargs["session"], resources=["ec2"], region=kwargs["region"]
    )

    logger.info("Discovering instances")

    filters = build_filters(**kwargs)
    instances = search_instances(filters, session_objects, **kwargs)

    if kwargs.get("invert"):
        logger.debug("Inverting the results")
        all_instances = session_objects["ec2_resource"].instances.all()
        instances = [
            instance.id for instance in all_instances if instance.id not in instances
        ]

    logger.info(f"{len(instances)} instances found")
    logger.debug(instances)

    return instances


def build_filters(**kwargs):
    """Construct filters"""

    tags = kwargs.get("tags")
    public = kwargs.get("public")
    states = kwargs.get("states")
    instance_ids = kwargs.get("instance_ids")

    filters = []
    filters.append({"Name": "instance-state-name", "Values": states})

    if tags:
        logger.debug(f"Filtering for tags: {tags}")
        for tag in tags:
            parsed_tag = parse_tag(tag)
            if parsed_tag[1]:
                filters.append(
                    {"Name": f"tag:{parsed_tag[0]}", "Values": [parsed_tag[1]]}
                )
            else:
                filters.append({"Name": "tag-key", "Values": [parsed_tag[0]]})

    if public:
        logger.debug("Filtering for public instances")
        filters.append({"Name": "ip-address", "Values": ["*"]})

    if instance_ids:
        logger.debug(f"Filtering for instances: {instance_ids}")

    if kwargs.get("os") and kwargs["os"].lower() == "windows":
        logger.debug(f"Filtering for OS: {kwargs['os']}")
        filters.append({"Name": "platform", "Values": ["windows"]})

    return filters


def search_instances(filters, session_objects, **kwargs):
    """Search for instances in specified region"""

    response = session_objects["ec2_resource"].instances.filter(
        Filters=filters, InstanceIds=kwargs["instance_ids"]
    )
    instances = [instance.id for instance in response]

    if kwargs.get("os") and kwargs["os"].lower() == "linux":
        windows_filters = [{"Name": "platform", "Values": ["windows"]}]
        response = session_objects["ec2_resource"].instances.filter(
            Filters=windows_filters, InstanceIds=kwargs["instance_ids"]
        )
        windows_instances = [instance.id for instance in response]
        instances = [id for id in instances if id not in windows_instances]

    return instances


def parse_tag(tag):
    if "=" in tag:
        return tag.split("=", 1)
    else:
        return tag, None



def set_defaults(kwargs, default_values):
    """Set default values for input parameters."""

    if default_values is None:
        return kwargs
    for key, default in default_values.items():
        kwargs[key] = default if kwargs.get(key) is None else kwargs[key]

    return kwargs



from avtomat_aws.services.ec2 import discover_active_regions

ACTION_DESCRIPTION = "Discover active regions for an account."


def add_cli_arguments(parser):
    """Argument parsing"""
    pass


def cli(args):
    """Command-line interface function"""

    inputs = vars(args)

    try:
        result = discover_active_regions(**inputs)
        for item in result:
            print(item)
    except Exception as e:
        print(f"Action failed - {e}")
        exit(1)



import logging

from avtomat_aws.decorators.authenticate import authenticate
from avtomat_aws.decorators.set_logger import set_logger
from avtomat_aws.decorators.validate import validate
from avtomat_aws.helpers.set_session_objects import set_session_objects

logger = logging.getLogger(__name__)

DEFAULTS = {
    "enable": False,
    "disable": False,
    "region": None,
    "debug": False,
    "silent": False,
}
RULES = [
    {"required": ["users"]},
    {"at_least_one": ["enable", "disable"]},
    {"at_most_one": ["enable", "disable"]},
]


@validate(DEFAULTS, RULES)
@set_logger()
@authenticate()
def modify_users_console_access(**kwargs):
    """Enable or disable AWS Management Console access for an IAM user"""

    # Needs a validation rule
    if kwargs.get("enable"):
        missing_passwords = [
            user for user in kwargs["users"] if not user.get("Password")
        ]
        if missing_passwords:
            missing_usernames = ", ".join(
                [user["UserName"] for user in missing_passwords]
            )
            raise ValueError(
                f"Password is required when enabling console access for users: {missing_usernames}"
            )

    # Required parameters
    users = kwargs.pop("users")

    if not users:
        logger.info("No users to modify")
        return []

    session_objects = set_session_objects(
        kwargs["session"], clients=["iam"], region=kwargs["region"]
    )

    if kwargs.get("enable"):
        logger.info(f"Enabling console access for users")
    elif kwargs.get("disable"):
        logger.info(f"Disabling console access for users")
    logger.debug(f"Users: {users}")

    failed = []
    counter = 0
    if kwargs.get("enable"):
        for user in users:
            username = user["UserName"]
            password = user["Password"]
            try:
                session_objects["iam_client"].create_login_profile(
                    UserName=username, Password=password, PasswordResetRequired=True
                )
                session_objects["iam_client"].attach_user_policy(
                    UserName=username,
                    PolicyArn="arn:aws:iam::aws:policy/IAMUserChangePassword",
                )
                counter += 1
            except Exception as e:
                failed.append(username)
                logger.error(f"{username} - failed to enable console access - {e}")
                continue
    elif kwargs.get("disable"):
        for user in users:
            username = user["UserName"]
            try:
                session_objects["iam_client"].delete_login_profile(UserName=username)
                counter += 1
            except Exception as e:
                failed.append(username)
                logger.error(f"{username} - failed to disable console access - {e}")
                continue

    if kwargs.get("enable"):
        logger.info(f"Enabled console access for {counter} users")
    elif kwargs.get("disable"):
        logger.info(f"Disabled console access for {counter} users")
    if failed:
        logger.warning(f"{len(failed)} users failed to update")
        logger.debug(failed)

    return failed



from avtomat_aws.helpers.set_region import set_region
from avtomat_aws.helpers.set_session import set_session


def authenticate():
    """Decorator to authenticate with AWS and set a region for the action."""

    def decorator(func):
        def wrapper(**kwargs):
            # Create a session
            kwargs["session"] = set_session(**kwargs)
            # Set the region for the session
            kwargs["region"] = set_region(
                region=kwargs.get("region"),
                session=kwargs.get("session"),
                debug=kwargs.get("debug"),
                silent=kwargs.get("silent"),
            )
            # Now that 'session' and 'region' are ensured, call the action
            return func(**kwargs)

        return wrapper

    return decorator



import logging

from avtomat_aws.decorators.authenticate import authenticate
from avtomat_aws.decorators.set_logger import set_logger
from avtomat_aws.decorators.validate import validate
from avtomat_aws.helpers.set_session_objects import set_session_objects

logger = logging.getLogger(__name__)

DEFAULTS = {"re_encrypt": False, "region": None, "debug": False, "silent": False}
RULES = [{"required": ["volume_id", "kms_key_id"]}]


@validate(DEFAULTS, RULES)
@set_logger()
@authenticate()
def encrypt_volume(**kwargs):
    """Encrypt an EBS volume with a KMS key"""

    # Required parameters
    volume_id = kwargs.pop("volume_id")
    kms_key_id = kwargs.pop("kms_key_id")

    session_objects = set_session_objects(
        kwargs.get("session"),
        clients=["ec2"],
        resources=["ec2"],
        region=kwargs["region"],
    )

    logger.info(f"{volume_id} - encrypting with {kms_key_id}")

    # Create snapshot
    volume = session_objects["ec2_resource"].Volume(volume_id)

    if not kwargs.get("re_encrypt") and volume.encrypted:
        logger.info(f"{volume_id} - skipped, already encrypted")
        logger.info("Done")
        return volume_id

    attached = False
    if volume.state == "in-use":
        attached = True
        instance_id = volume.attachments[0]["InstanceId"]
        device = volume.attachments[0]["Device"]
        instance = session_objects["ec2_resource"].Instance(instance_id)

        # Check instance state to ensure it's started only if originally running
        originally_running = False
        if instance.state["Name"] == "running":
            originally_running = True

    snapshot = session_objects["ec2_resource"].create_snapshot(
        VolumeId=volume_id,
        Description=f"Pre-encryption snapshot of {volume_id}",
        TagSpecifications=[
            {
                "ResourceType": "snapshot",
                "Tags": [{"Key": "Name", "Value": f"source:{volume_id}"}],
            }
        ],
    )
    snapshot.load()
    logger.info(f"{snapshot.id} - creating")
    snapshot.wait_until_completed()

    # Create encrypted volume from snapshot
    encrypted_volume = session_objects["ec2_resource"].create_volume(
        SnapshotId=snapshot.id,
        AvailabilityZone=volume.availability_zone,
        Encrypted=True,
        KmsKeyId=kms_key_id,
    )
    logger.info(f"{encrypted_volume.id} - creating")
    waiter = session_objects["ec2_client"].get_waiter("volume_available")
    waiter.wait(VolumeIds=[encrypted_volume.id])

    # Copy tags from original volume to encrypted volume
    original_tags = volume.tags
    if original_tags is None:
        original_tags = []
    if original_tags:
        session_objects["ec2_client"].create_tags(
            Resources=[encrypted_volume.id], Tags=original_tags
        )
    logger.info(f"{encrypted_volume.id} - tags copied from {volume_id}")

    # If original volume is in use, swap volumes
    if attached:
        instance.stop()
        logger.info(f"{instance_id} - stopping")
        instance.wait_until_stopped()

        logger.info(f"{volume_id} ({device}) - detaching")
        volume.detach_from_instance()
        waiter = session_objects["ec2_client"].get_waiter("volume_available")
        waiter.wait(VolumeIds=[volume_id])

        logger.info(f"{encrypted_volume.id} - attaching to {instance_id} as {device}")
        instance.attach_volume(VolumeId=encrypted_volume.id, Device=device)

        if originally_running:
            instance.start()
            logger.info(f"{instance_id} - starting")
            instance.wait_until_running()

    # Cleanup
    snapshot.delete()
    logger.info(f"{snapshot.id} - deleted")

    # Rollback instructions
    if attached:
        logger.info(
            f"To rollback, attach {volume_id} to {instance_id} as {device} and delete {encrypted_volume.id}"
        )
    else:
        logger.info(f"To rollback, delete {encrypted_volume.id}")

    return encrypted_volume.id



from avtomat_aws.services.iam import quarantine_user

ACTION_DESCRIPTION = "Disable console and programmatic access and apply AWSCompromisedKeyQuarantineV2 policy to an IAM user."


def add_cli_arguments(parser):
    """Argument parsing"""

    required = parser.add_argument_group("required")

    required.add_argument("--user", help="User to quarantine.", required=True)


def cli(args):
    """Command-line interface function"""

    inputs = vars(args)

    try:
        quarantine_user(**inputs)
    except Exception as e:
        print(f"Action failed - {e}")
        exit(1)



import logging
import time

from avtomat_aws.decorators.authenticate import authenticate
from avtomat_aws.decorators.set_logger import set_logger
from avtomat_aws.decorators.validate import validate
from avtomat_aws.helpers.set_session_objects import set_session_objects

logger = logging.getLogger(__name__)

DEFAULTS = {
    "backup_vault_name": "Default",
    "retention_days": 14,
    "iam_role": None,
    "wait": False,
    "region": None,
    "debug": False,
    "silent": False,
}
RULES = [
    {"required": ["resource_ids", "service"]},
    {"choice": [{"service": ["ec2", "ebs"]}]},
]


@validate(DEFAULTS, RULES)
@set_logger()
@authenticate()
def create_backups(**kwargs):
    """Start an on-demand backup job for the specified resources"""

    # Required parameters
    resource_ids = kwargs.pop("resource_ids")
    service = kwargs.pop("service")

    if not resource_ids:
        logger.info("No resources to backup")
        return []

    session_objects = set_session_objects(
        kwargs["session"], clients=["backup", "sts"], region=kwargs["region"]
    )

    current_account = session_objects["sts_client"].get_caller_identity()["Account"]
    if not kwargs.get("iam_role"):
        kwargs["iam_role"] = (
            f"arn:aws:iam::{current_account}:role/service-role/AWSBackupDefaultServiceRole"
        )

    logger.info(f"Creating backups for resources")

    failed_jobs = []
    initiated_jobs = {}
    for resource_id in resource_ids:
        try:
            resource_arn = construct_arn(
                service, resource_id, kwargs["region"], current_account
            )
            response = session_objects["backup_client"].start_backup_job(
                BackupVaultName=kwargs["backup_vault_name"],
                ResourceArn=resource_arn,
                IamRoleArn=kwargs["iam_role"],
                StartWindowMinutes=60,
                Lifecycle={"DeleteAfterDays": kwargs["retention_days"]},
            )
            initiated_jobs[response["BackupJobId"]] = resource_id
        except Exception as e:
            logger.error(f"Failed to create a backup for {resource_id} - {e}")
            failed_jobs.append(resource_id)
            continue

    if kwargs.get("wait"):
        logger.info("Waiting for backup jobs to complete")
        failed_jobs = wait_backups(
            session_objects["backup_client"], initiated_jobs, failed_jobs
        )

    if failed_jobs:
        logger.info(f"{len(failed_jobs)} resources failed backup creation")
    else:
        logger.info("Success")
    logger.debug(failed_jobs)

    return failed_jobs


def wait_backups(backup_client, initiated_jobs, failed_jobs):
    """Wait for backup jobs to complete"""

    while initiated_jobs:
        for job_id, resource_id in list(initiated_jobs.items()):
            try:
                response = backup_client.describe_backup_job(BackupJobId=job_id)
                status = response["State"]
                if status in ["COMPLETED"]:
                    del initiated_jobs[job_id]
                elif status in ["FAILED", "ABORTED"]:
                    del initiated_jobs[job_id]
                    failed_jobs.append(resource_id)
            except Exception as e:
                logger.error(f"Failed to get status for backup of {resource_id} - {e}")
                continue

        if initiated_jobs:
            logger.debug(f"Waiting for {len(initiated_jobs)} backup jobs to complete")
            time.sleep(60)

    return failed_jobs


def construct_arn(service, resource_id, region, account_id):
    """Construct ARN for the specified resource"""

    if service == "ec2":
        return f"arn:aws:ec2:{region}:{account_id}:instance/{resource_id}"
    if service == "ebs":
        return f"arn:aws:ec2:{region}:{account_id}:volume/{resource_id}"



import logging

from avtomat_aws.decorators.authenticate import authenticate
from avtomat_aws.decorators.set_logger import set_logger
from avtomat_aws.decorators.validate import validate
from avtomat_aws.helpers.set_session_objects import set_session_objects

logger = logging.getLogger(__name__)

DEFAULTS = {"snapshot": False, "region": None, "debug": False, "silent": False}
RULES = [{"required": ["volume_ids"]}]


@validate(DEFAULTS, RULES)
@set_logger()
@authenticate()
def delete_volumes(**kwargs):
    """Delete EBS volumes"""

    # Required parameters
    volume_ids = kwargs.pop("volume_ids")

    if not volume_ids:
        logger.info("No volumes to delete")
        return []

    session_objects = set_session_objects(
        kwargs["session"], resources=["ec2"], region=kwargs["region"]
    )

    logger.info(f"Deleting volumes")
    if kwargs.get("snapshot"):
        logger.info("Creating snapshots before deletion")

    snapshots_to_wait = []
    for volume_id in volume_ids:
        volume = session_objects["ec2_resource"].Volume(volume_id)
        if volume.state == "in-use":
            logger.warning(f"{volume_id} - in use, skipping")
            continue

        if kwargs.get("snapshot") and volume.state == "available":
            try:
                snapshot = volume.create_snapshot(
                    Description=f"Backup before deletion of {volume_id}",
                    TagSpecifications=[
                        {
                            "ResourceType": "snapshot",
                            "Tags": [{"Key": "Name", "Value": f"source:{volume_id}"}],
                        }
                    ],
                )
                logger.info(f"{volume_id} - creating {snapshot.id}")
                snapshots_to_wait.append(snapshot)
            except Exception as e:
                logger.error(f"{volume_id} - failed to create snapshot - {e}")
                continue

    for snapshot in snapshots_to_wait:
        snapshot.wait_until_completed()

    failed = []
    counter = 0
    for volume_id in volume_ids:
        try:
            volume = session_objects["ec2_resource"].Volume(volume_id)
            if volume.state != "available":
                continue
            volume.delete()
            counter += 1
            logger.info(f"{volume_id} - deleted")
        except Exception as e:
            failed.append(volume_id)
            logger.error(f"{volume_id} - failed to delete - {e}")
            continue

    logger.info(f"{counter} volumes deleted")
    if failed:
        logger.warning(f"{len(failed)} volumes failed to delete")
        logger.debug(failed)

    return failed



from .copy_snapshots import copy_snapshots
from .create_snapshots import create_snapshots
from .delete_images import delete_images
from .delete_security_groups import delete_security_groups
from .delete_snapshots import delete_snapshots
from .delete_volumes import delete_volumes
from .discover_active_regions import discover_active_regions
from .discover_default_ebs_encryption import discover_default_ebs_encryption
from .discover_images import discover_images
from .discover_instances import discover_instances
from .discover_no_ssm_instances import discover_no_ssm_instances
from .discover_snapshots import discover_snapshots
from .discover_tags import discover_tags
from .discover_unused_security_groups import discover_unused_security_groups
from .discover_volumes import discover_volumes
from .encrypt_instance_volumes import encrypt_instance_volumes
from .encrypt_volume import encrypt_volume
from .modify_default_ebs_encryption import modify_default_ebs_encryption
from .modify_tags import modify_tags
from .modify_volumes import modify_volumes
from .share_snapshots import share_snapshots



from avtomat_aws.services.ec2 import discover_snapshots

ACTION_DESCRIPTION = "Discover snapshots of EBS volumes."


def add_cli_arguments(parser):
    """Argument parsing"""

    parser.add_argument(
        "--volume_ids",
        nargs="+",
        help="Get snapshots that originate from specific volumes.",
        required=False,
    )
    parser.add_argument(
        "--snapshot_ids",
        nargs="+",
        help="Snapshot IDs to focus on, separated by space.",
        required=False,
    )
    parser.add_argument(
        "--unencrypted",
        action="store_true",
        help="Get only unencrypted snapshots.",
        required=False,
    )
    parser.add_argument(
        "--exclude_aws_backup",
        action="store_true",
        help="Exclude snapshots managed by AWS Backup.",
        required=False,
    )
    parser.add_argument(
        "--created_before",
        help="Get snapshots created before date (YYYY/MM//DD).",
        required=False,
    )
    parser.add_argument(
        "--created_after",
        help="Get snapshots created after date (YYYY/MM//DD).",
        required=False,
    )


def cli(args):
    """Command-line interface function"""

    inputs = vars(args)

    try:
        result = discover_snapshots(**inputs)
        for item in result:
            print(item)
    except Exception as e:
        print(f"Action failed - {e}")
        exit(1)



from avtomat_aws.services.ec2 import discover_tags

ACTION_DESCRIPTION = "Find existing or missing tags on EC2 resources."


def add_cli_arguments(parser):
    """Argument parsing"""

    required = parser.add_argument_group("required")

    required.add_argument(
        "--resource_types",
        nargs="+",
        help="List of EC2 resource types to search through.",
        required=True,
    )
    required.add_argument(
        "--tags",
        nargs="+",
        help="Tags in 'Key=Value' or 'Key' format to search for.",
        required=True,
    )
    parser.add_argument(
        "--existing",
        action="store_true",
        help="Search for resources that have the tags.",
        required=False,
    )
    parser.add_argument(
        "--missing",
        action="store_true",
        help="Search for resources that don't have the tags.",
        required=False,
    )


def cli(args):
    """Command-line interface function"""

    inputs = vars(args)

    try:
        result = discover_tags(**inputs)
        for item in result:
            print(item)
    except Exception as e:
        print(f"Action failed - {e}")
        exit(1)
